/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda;

import java.io.PrintStream;

import org.xmloperator.lambda.net.impl.OperatorFactoryImpl;
import org.xmloperator.lambda.net.util.InteractionStat;
import org.xmloperator.lambda.shared.ObjectArrayFactoryImpl;
import org.xmloperator.lambda.tree.impl.TermFactoryImpl;
import org.xmloperator.lambda.tree.serialize.PrefixedSerializer;
import org.xmloperator.lambda.tree.serialize.Serializer;
import org.xmloperator.lambda.tree.serialize.UsualSerializer;

/**
 * Test package.
 */
public abstract class MainTest {
	public static final boolean ASSERTION = false;
	public static final boolean SCOPE_BINDING = true;
	private static final boolean REUSE = true;
	private static final String SERIALIZER_TYPE = "Prefixed";
	private static final boolean IS_VERBOSE = false;

	private static final ObjectArrayFactoryImpl OBJECT_ARRAY_FACTORY =
		new ObjectArrayFactoryImpl();
	public static final TermFactoryImpl TERM_FACTORY =
		new TermFactoryImpl(REUSE, OBJECT_ARRAY_FACTORY);
	public static final OperatorFactoryImpl OPERATOR_FACTORY =
		new OperatorFactoryImpl(REUSE, OBJECT_ARRAY_FACTORY);
	public static final Serializer SERIALIZER =
		SERIALIZER_TYPE == "Prefixed"
			? (Serializer) new PrefixedSerializer()
			: SERIALIZER_TYPE == "Usual"
			? (Serializer) new UsualSerializer(false)
			: null;
	public static final InteractionStat INTERACTION_STAT = new InteractionStat();

	public static void main(String[] args) {
		long t0 = System.currentTimeMillis();
		boolean isVerbose = IS_VERBOSE;
		if (test(System.out, isVerbose)) {
			long t1 = System.currentTimeMillis();
			System.out.println("All is OK, using " + (t1 - t0) + " ms.");
		}
		System.exit(0);
	}

	/**
	 * Executes some tests.
	 * @param out the PrintStream to print results.
	 * @param isVerbose if true then all tests have to be displayed.
	 * @return true if all is Ok.
	 */
	public static boolean test(PrintStream out, boolean isVerbose) {
		boolean isOk = true;
		isOk &= org.xmloperator.lambda.tree.impl.Test.test(out, isVerbose);
		isOk &= org.xmloperator.lambda.tree.util.Test.test(out, isVerbose);
		isOk &= org.xmloperator.lambda.tree.serialize.Test.test(out, isVerbose);
		isOk &= org.xmloperator.lambda.tree.reduction.Test.test(out, isVerbose);
		isOk &= org.xmloperator.lambda.tree.generate.Test.test(out, isVerbose);
		isOk &= org.xmloperator.lambda.net.impl.Test.test(out, isVerbose);
		isOk &= org.xmloperator.lambda.net.util.Test.test(out, isVerbose);
		isOk &= org.xmloperator.lambda.net.reduction.Test.test(out, isVerbose);
		isOk &= org.xmloperator.lambda.translate.Test.test(out, isVerbose);
		if (isVerbose || (REUSE && !OBJECT_ARRAY_FACTORY.isWellBalanced()))
			System.out.println(OBJECT_ARRAY_FACTORY);
		if (isVerbose || (REUSE && !TERM_FACTORY.isWellBalanced()))
			System.out.println(TERM_FACTORY);
		if (isVerbose || (REUSE && !OPERATOR_FACTORY.isWellBalanced()))
			System.out.println(OPERATOR_FACTORY);
		if (isVerbose)
			System.out.println(
				INTERACTION_STAT.report(InteractionStat.REPORT_STYLE_CLASS));
		return isOk;
	}
}
