/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.net.impl;

import org.xmloperator.lambda.net.exception.UnexpectedAnnihilationException;
import org.xmloperator.lambda.net.model.Abstractor;
import org.xmloperator.lambda.net.model.ConnectionListener;
import org.xmloperator.lambda.net.model.Operator;
import org.xmloperator.lambda.net.model.OperatorList;
import org.xmloperator.lambda.net.model.Port;
import org.xmloperator.lambda.net.model.ScopeBinder;
import org.xmloperator.lambda.shared.ObjectArrayFactory;

/**
 * Abstractor implementation.
 */
final class AbstractorImpl extends OperatorImpl implements Abstractor {
	final PortImpl input;
	final PortImpl body;
	final PortImpl bind;
	ScopeBinder scopeBinder = null;

	AbstractorImpl(
		OperatorFactoryImpl operatorFactory,
		ObjectArrayFactory objectArrayFactory,
		int mark) {
		super(operatorFactory, mark);
		this.input = new PortImpl(Port.TYPE_ABSTRACTOR_INPUT, this, 0, true);
		this.body = new PortImpl(Port.TYPE_ABSTRACTOR_BODY, this, 1, false);
		this.bind = new PortImpl(Port.TYPE_ABSTRACTOR_BIND, this, 2, true);
	}

	//////////////////////////
	// Operator implementation
	//////////////////////////

	public boolean canBeReturnedToFactory() {
		return super.canBeReturnedToFactory()
			&& this.input.connectedPort == null
			&& this.body.connectedPort == null
			&& this.bind.connectedPort == null;
	}

	public void returnToFactory() {
		super.returnToFactory();
		this.detachScopeBinder();
		this.operatorFactory.returnAbstractor(this);
	}

	public int getPortCount() {
		return 3;
	}

	public int getOperatorType() {
		return Operator.TYPE_ABSTRACTOR;
	}

	public boolean isAbstractor() {
		return true;
	}

	public Port getPort(int number) {
		switch (number) {
			case 0 :
				return this.input;
			case 1 :
				return this.body;
			case 2 :
				return this.bind;
			default :
				throw new IllegalArgumentException();
		}
	}

	public Port getPort0() {
		return this.input;
	}

	public Port getPort1() {
		return this.body;
	}

	public Port getPort2() {
		return this.bind;
	}

	public boolean isDirectionInversed() {
		return false;
	}

	public Operator cloneOperator(boolean isDirectionInversed) {
		if (isDirectionInversed)
			throw new IllegalArgumentException();
		return this.operatorFactory.createAbstractor(this.mark, this.scopeBinder);
	}

	public int interact(ConnectionListener xListener) {
		Operator connectedOperator =
			this.getPort0().getConnectedPort().getOperator();
		if (connectedOperator.isApplicator())
			return Beta.interact((ApplicatorImpl) connectedOperator, this, xListener);
		if (connectedOperator.isAbstractor())
			throw new UnexpectedAnnihilationException(this);
		return connectedOperator.interact(xListener);
	}

	////////////////////////////
	// Abstractor implementation
	////////////////////////////

	public Port getInput() {
		return this.input;
	}

	public Port getBody() {
		return this.body;
	}

	public Port getBind() {
		return this.bind;
	}

	public boolean isIdentity() {
		return this.body.connectedPort == this.bind;
	}

	public ScopeBinder getScopeBinder() {
		return this.scopeBinder;
	}

	public ScopeBinder attachScopeBinder() {
		if (this.scopeBinder == null) {
			this.scopeBinder = this.operatorFactory.createScopeBinder();
			this.scopeBinder.getAbstractorList().add(this);
		}
		return this.scopeBinder;
	}

	public void detachScopeBinder() {
		if (this.scopeBinder != null) {
			OperatorList abstractorList = this.scopeBinder.getAbstractorList();
			abstractorList.remove(this);
			if (abstractorList.isEmpty())
				this.scopeBinder.returnToFactory();
			this.scopeBinder = null;
		}
	}
}
