/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.net.impl;

import org.xmloperator.lambda.net.exception.AlreadyConnectedException;
import org.xmloperator.lambda.net.exception.ReturnedToFactoryException;
import org.xmloperator.lambda.net.exception.SameTypePortException;
import org.xmloperator.lambda.net.model.Operator;
import org.xmloperator.lambda.net.model.Port;

/**
 * Port implementation.
 */
final class PortImpl implements Port {
	private static final boolean ASSERTION = Test.ASSERTION;

	final int type;
	OperatorImpl operator;
	int number;
	boolean isInput;
	PortImpl connectedPort = null;

	PortImpl(int portType, OperatorImpl operator, int number, boolean isInput) {
		this.type = portType;
		this.operator = operator;
		this.number = number;
		this.isInput = isInput;
	}

	//////////////////////
	// Port implementation
	//////////////////////

	public final int getPortType() {
		return this.type;
	}

	public final Operator getOperator() {
		return this.operator;
	}

	public final int getNumber() {
		return this.number;
	}

	public boolean isPrincipal() {
		return this.number == 0;
	}

	public boolean isFirstAuxiliary() {
		return this.number == 1;
	}

	public boolean isSecondAuxiliary() {
		return this.number == 2;
	}

	public boolean isInput() {
		return this.isInput;
	}

	public final Port getConnectedPort() {
		return this.connectedPort;
	}

	public final void connectTo(Port connectedPort) {
		PortImpl oldConnectedPort = this.connectedPort;
		PortImpl newConnectedPort = (PortImpl) connectedPort;
		if (ASSERTION) {
			if (connectedPort == null || connectedPort == this)
				throw new IllegalArgumentException();
			if (this.operator.isReturnedToTheFactory)
				throw new ReturnedToFactoryException(this.operator);
			if (newConnectedPort.operator.isReturnedToTheFactory)
				throw new ReturnedToFactoryException(newConnectedPort.operator);
			if (oldConnectedPort != null)
				throw new AlreadyConnectedException(oldConnectedPort);
			if (newConnectedPort.connectedPort != null)
				throw new AlreadyConnectedException(newConnectedPort.connectedPort);
			if (newConnectedPort.isInput == this.isInput)
				throw new SameTypePortException(newConnectedPort);
		}
		newConnectedPort.connectedPort = this;
		this.connectedPort = newConnectedPort;
	}

	public final void disconnect() {
		PortImpl oldConnectedPort = this.connectedPort;
		if (oldConnectedPort != null) {
			oldConnectedPort.connectedPort = null;
			this.connectedPort = null;
		}
	}
}
