/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.net.impl;

import org.xmloperator.lambda.net.model.Operator;
import org.xmloperator.lambda.net.model.Port;
import org.xmloperator.lambda.net.model.PortStack;
import org.xmloperator.lambda.shared.BaseStack;
import org.xmloperator.lambda.shared.ObjectArrayFactory;

/**
 * PortStack implementation.
 */
public final class PortStackImpl extends BaseStack implements PortStack {
	private static final boolean ASSERTION = Test.ASSERTION;
	final OperatorFactoryImpl operatorFactory;

	PortStackImpl(
		OperatorFactoryImpl operatorFactory,
		ObjectArrayFactory objectArrayFactory) {
		super(objectArrayFactory, false);
		this.operatorFactory = operatorFactory;
	}

	////////////////////////////////////
	// ConnectionListener implementation
	////////////////////////////////////

	public void notifyConnection(Port port1, Port port2) {
		if (!port1.isPrincipal() || !port2.isPrincipal())
			return;
		{
			Operator operator1 = port1.getOperator();
			Operator operator2 = port2.getOperator();
			if (operator1.isAbstractor()) {
				if (operator2.isApplicator())
					return;
			}
			else if (operator1.isApplicator()) {
				if (operator2.isAbstractor())
					return;
			}
		}
		if (ASSERTION
			&& (this.containsObject(port1) || this.containsObject(port2)))
			throw new IllegalStateException();
		this.pushObject(port1); // One port is sufficient.
	}

	///////////////////////////
	// PortStack implementation
	///////////////////////////

	public void returnToFactory() {
		this.endUse();
		this.operatorFactory.returnPortStack(this);
	}

	public void push(Port port) {
		this.pushObject(port);
	}

	public Port pop() {
		return (Port) this.popObject();
	}

	public Port top() {
		return (Port) this.topObject();
	}

	public Port get(int index) {
		return (Port) this.objects[index];
	}

	public void replace(int index, Port port) {
		this.objects[index] = port;
	}

	public boolean contains(Port port) {
		return this.containsObject(port);
	}

	public boolean remove(Port port) {
		return this.removeObject(port);
	}
}
