/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.net.impl;

import org.xmloperator.lambda.net.model.OperatorList;
import org.xmloperator.lambda.net.model.ScopeBinder;
import org.xmloperator.lambda.shared.ObjectArrayFactory;

/**
 * ScopeBinder implementation.
 */
public final class ScopeBinderImpl implements ScopeBinder {
	final OperatorFactoryImpl operatorFactory;
	boolean isReturnedToTheFactory = false;
	private final OperatorListImpl abstractorList;
	private final OperatorListImpl delimiterList;

	public ScopeBinderImpl(
		OperatorFactoryImpl operatorFactory,
		ObjectArrayFactory objectArrayFactory) {
		this.operatorFactory = operatorFactory;
		this.abstractorList = new OperatorListImpl(objectArrayFactory);
		this.delimiterList = new OperatorListImpl(objectArrayFactory);
	}

	void startUse() {
		this.abstractorList.startUse();
		this.delimiterList.startUse();
	}

	/////////////////////////////
	// ScopeBinder implementation
	/////////////////////////////

	public void returnToFactory() {
		{
			int abstractorCount = this.abstractorList.getSize();
			for (int k = 0; k < abstractorCount; k++)
				 ((AbstractorImpl) this.abstractorList.get(k)).scopeBinder = null;
		}
		{
			int delimiterCount = this.delimiterList.getSize();
			for (int k = 0; k < delimiterCount; k++)
				 ((DelimiterImpl) this.delimiterList.get(k)).scopeBinder = null;
		}
		this.abstractorList.endUse();
		this.delimiterList.endUse();
		this.operatorFactory.returnScopeBinder(this);
	}

	public OperatorList getAbstractorList() {
		return this.abstractorList;
	}

	public OperatorList getDelimiterList() {
		return this.delimiterList;
	}
}
