/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.net.impl;

import org.xmloperator.lambda.net.model.Walk;
import org.xmloperator.lambda.net.model.WalkStack;

/**
 * Walk implementation.
 */
final class WalkImpl implements Walk, WalkStack {
	final OperatorFactoryImpl operatorFactory;
	boolean isReturnedToTheFactory = false;
	int bindIndex;
	DirectorStep directorStep = null;
	WalkStack walkStack = null;

	WalkImpl(OperatorFactoryImpl operatorFactory, int bindIndex) {
		this.operatorFactory = operatorFactory;
		this.bindIndex = bindIndex;
	}

	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append(bindIndex);
		{
			DirectorStep directorStep = this.directorStep;
			while (directorStep != null) {
				sb.append(directorStep.director == 1 ? 'L' : 'R');
				directorStep = directorStep.nextStep;
			}
		}
		if (this.walkStack != null)
			sb.append(this.walkStack);
		return sb.toString();
	}

	//////////////////////
	// Walk implementation
	//////////////////////

	public void returnToFactory() {
		while (this.walkStack != null)
			this.pop().returnToFactory();
		while (this.directorStep != null)
			this.popDirector();
		this.isReturnedToTheFactory = true;
		this.operatorFactory.returnWalk(this);
	}

	public int getBindIndex() {
		return this.bindIndex;
	}

	public void pushDirector(int director) {
		if (this.isReturnedToTheFactory)
			throw new IllegalStateException();
		this.directorStep =
			this.operatorFactory.createDirectorStep(director, this.directorStep);
	}

	public int popDirector() {
		if (this.directorStep == null)
			throw new IllegalStateException();
		DirectorStep directorStep = this.directorStep;
		int director = directorStep.director;
		this.directorStep = directorStep.nextStep;
		this.operatorFactory.returnDirectorStep(directorStep);
		return director;
	}

	public WalkStack getWalkStack() {
		return this;
	}

	///////////////////////////
	// WalkStack implementation
	///////////////////////////

	public int getSize() {
		WalkStack walkStack = this.walkStack;
		return walkStack != null ? walkStack.getSize() : 0;
	}

	public boolean isEmpty() {
		WalkStack walkStack = this.walkStack;
		return walkStack != null ? walkStack.isEmpty() : true;
	}

	public void push(Walk walk) {
		if (this.isReturnedToTheFactory)
			throw new IllegalStateException();
		WalkStack walkStack = this.walkStack;
		if (walkStack == null) {
			walkStack = this.operatorFactory.createWalkStack();
			this.walkStack = walkStack;
		}
		walkStack.push(walk);
	}

	public Walk pop() {
		WalkStack walkStack = this.walkStack;
		if (walkStack == null)
			throw new IllegalStateException();
		Walk walk = walkStack.pop();
		if (walkStack.isEmpty()) {
			walkStack.returnToFactory();
			this.walkStack = null;
		}
		return walk;
	}

	public Walk top() {
		WalkStack walkStack = this.walkStack;
		return walkStack != null ? walkStack.top() : null;
	}

	public Walk getFromTop(int index) {
		WalkStack walkStack = this.walkStack;
		if (walkStack == null)
			throw new IndexOutOfBoundsException();
		return walkStack.getFromTop(index);
	}

	public void insertFromTop(int index, Walk walk) {
		if (this.isReturnedToTheFactory)
			throw new IllegalStateException();
		WalkStack walkStack = this.walkStack;
		if (walkStack == null)
			throw new IndexOutOfBoundsException();
		walkStack.insertFromTop(index, walk);
	}

	public Walk removeFromTop(int index) {
		WalkStack walkStack = this.walkStack;
		if (walkStack == null)
			throw new IndexOutOfBoundsException();
		Walk walk = walkStack.removeFromTop(index);
		if (walkStack.isEmpty()) {
			walkStack.returnToFactory();
			this.walkStack = null;
		}
		return walk;
	}
}
