/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.net.model;

/**
 * Abstractor in a lambda-net.
 */
public interface Abstractor extends Operator {

	/**
	 * Returns its input Port.
	 * @return its input Port. This is the principal Port.
	 */
	public Port getInput();
	
	/**
	 * Returns its body Port.
	 * @return its body Port. This is the first auxiliary Port.
	 *   This is an output Port.
	 */
	public Port getBody();
	
	/**
	 * Returns its bind Port.
	 * @return its bind Port. This is the second auxiliary Port.
	 *   This is an input Port.
	 */
	public Port getBind();
	
	/**
	 * Returns true if the body Port is currently connected to the bind Port.
	 * @return true if the body Port is currently connected to the bind Port.
	 */
	public boolean isIdentity();

	/**
	 * Returns the ScopeBinder attached to this Abstractor. If it returns null
	 *   then this Abstractor is closed, the reciprocal is not true.
	 * @return the ScopeBinder attached to this Abstractor, if any.
	 */
	public ScopeBinder getScopeBinder();
	
	/**
	 * If this Abstractor is not already attached to a ScopeBinder then create
	 *   a ScopeBinder and attach it to the Abstractor.
	 * @return the ScopeBinder attached to this Abstractor.
	 */
	public ScopeBinder attachScopeBinder();
	
	/**
	 * Detachs its ScopeBinder, if any. If the ScopeBinder is no more attached to
	 *   any Abstractor then it is returned to the factory.
	 */
	public void detachScopeBinder();
}
