/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.net.model;

/**
 * Delimiter in a lambda-net.
 */
public interface Delimiter extends IndexedOperator {

	/**
	 * Returns its input Port (direction being not inversed). It is the principal
	 *   Port if the orientation is reversed, the first auxiliary Port
	 *   if the orientation is normal.
	 * @return its input Port (direction being not inversed).
	 */
	public Port getInput();
	
	/**
	 * Returns its body Port. It is the principal Port if the orientation is
	 *   normal, the first auxiliary Port if the orientation is reversed.
	 *   This is an output Port if the direction is not inversed.
	 * @return its body Port.
	 */
	public Port getBody();

	/**
	 * Reverse the orientation of the principal Port and clear the index.
	 * <p><b>Pre-condition:</b> the principal Port is the body Port (this is the
	 *   normal orientation).</p>
	 * <p><b>Post-condition:</b> the principal Port is the input Port (this is the
	 *   reversed orientation).</p>
	 * @throws IllegalStateException if the orientation is already reversed.
	 */
	public void reverseOrientation();

	/**
	 * Returns the ScopeBinder attached to this Abstractor, if any.
	 * @return the ScopeBinder attached to this Abstractor. May be null.
	 */
	public ScopeBinder getScopeBinder();
}
