/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.net.model;

/**
 * This is a factory of operators in a lambda-net.
 * Operators that are no more used can be returned to the factory; they can be
 *   recycled.
 */
public interface OperatorFactory {

	/**
	 * Creates and returns a new (active) Eraser.
	 * @param mark a mark.
	 * @param isDirectionInversed true for inverting direction.
	 * @param isActive true for an active Eraser, false for a root Eraser.
	 * @return the created Eraser.
	 */
	public Eraser createEraser(
		int mark,
		boolean isDirectionInversed,
		boolean isActive);

	/**
	 * Creates and returns a new Delimiter.
	 * @param mark a mark.
	 * @param isDirectionInversed true for inverting direction.
	 * @param scopeBinder a ScopeBinder. May be null.
	 * @return the created Delimiter.
	 */
	public Delimiter createDelimiter(
		int mark,
		boolean isDirectionInversed,
		ScopeBinder scopeBinder);

	/**
	 * Creates and returns a new Abstractor.
	 * @param mark a mark.
	 * @param scopeBinder a ScopeBinder. May be null.
	 * @return the created Abstractor.
	 */
	public Abstractor createAbstractor(int mark, ScopeBinder scopeBinder);

	/**
	 * Creates and returns a new Applicator.
	 * @param mark a mark.
	 * @return the created Applicator.
	 */
	public Applicator createApplicator(int mark);

	/**
	 * Creates and returns a new Duplicator.
	 * @param mark a mark.
	 * @param isDirectionInversed true for inverting direction.
	 * @return the created Duplicator.
	 */
	public Duplicator createDuplicator(int mark, boolean isDirectionInversed);

	/**
	 * Creates and returns a new PortStack.
	 * @return the created PortStack.
	 */
	public PortStack createPortStack();

	/**
	 * Creates a new ScopeBinder and returns it.
	 * @return the created ScopeBinder.
	 */
	public ScopeBinder createScopeBinder();

	/**
	 * Creates and returns a new Walk.
	 * @param bindIndex a bind index.
	 * @return the created Walk.
	 */
	public Walk createWalk(int bindIndex);

	/**
	 * Creates and returns a new WalkStack.
	 * @return the created WalkStack.
	 */
	public WalkStack createWalkStack();
}
