/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.net.model;

/**
 * List of Operators.
 */
public interface OperatorList {
	
	/**
	 * Return the size of the list.
	 * @return the size of the list.
	 */
	public int getSize();

	/**
	 * Returns <tt>true</tt> if this list is empty.
	 * @return <tt>true</tt> if this list is empty.
	 */
	public boolean isEmpty() ;

	/**
	 * Clear the content of this list.
	 */
	public void clear();
	
	/**
	 * Return the Operator at a given index in the list.
	 * @param index an index in the list, between 0 and size - 1.
	 * @return the Operator at the index position.
	 * @throws IndexOutOfBoundsException if index < 0 or index >= size.
	 */
	public Operator get(int index);
	
	/**
	 * Add an Operator to the list.
	 * @param operaror an Operator to add.
	 * @throws IllegalArgumentException if the given Operator is null.
	 */
	public void add(Operator operator);

	/**
	 * Remove an Operator from this list.
	 * @param operator an Operator.
	 * @return <tt>true</tt> if the Operator has been effectively removed.
	 */
	public boolean remove(Operator operator);
}
