/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.net.model;

/**
 * Stack of Port.
 * As a ConnectionListener, it keeps only the trace of principal, non beta,
 *   connections.
 */
public interface PortStack extends ConnectionListener {
	
	/**
	 * Return this PortStack to the Factory.
	 * <p><b>Post-condition:</b> this is no more usable.</p>
	 */
	public void returnToFactory();
	
	/**
	 * Returns the size of the stack.
	 * @return the size of the stack.
	 */
	public int getSize();

	/**
	 * Returns <tt>true</tt> if the stack is empty.
	 * @return <tt>true</tt> if the stack is empty.
	 */
	public boolean isEmpty() ;

	/**
	 * Clear the content of this stack.
	 */
	public void clear();
	
	/**
	 * Push a Port on the top of the stack.
	 * @param port a Port to push.
	 * @throws IllegalArgumentException if the given Port is null.
	 */
	public void push(Port port);

	/**
	 * Return the Port at the top and pop the stack.
	 * @return the Port at the top and pop the stack.
	 * @throws IllegalStateException if this stack is empty.
	 */
	public Port pop();

	/**
	 * Return the Port at the top or null is the stack is empty.
	 * @return the Port at the top or null if the stack is empty.
	 */
	public Port top();

	/**
	 * Return the Port at a given root relative index in the stack.
	 * <tt>get(size - 1)</tt> is equivalent to <tt>top()</tt>.
	 * @param index an index in the stack, between 0 and size - 1.
	 * @return the Port at the index position.
	 * @throws IndexOutOfBoundsException if index &lt; 0 or index &gt;= size.
	 */
	public Port get(int index);
	
	/**
	 * Replace the Port at a given root relative index in the stack.
	 * @param index an index in the stack, between 0 and size - 1.
	 * @param port a Port for replacing the Port actually at the given index.
	 * @throws IndexOutOfBoundsException if index &lt; 0 or index &gt;= size.
	 */
	public void replace(int index, Port port);

	/**
	 * Returns <tt>true</tt> if this stack contains the given Port.
	 * @param port a Port.
	 * @return <tt>true</tt> if this stack contains the given Port.
	 */
	public boolean contains(Port port);
	
	/**
	 * Removes a Port from this stack.
	 * @param port a Port.
	 * @return <tt>true</tt> if the Port has been effectively removed.
	 */
	public boolean remove(Port port);
}
