/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.net.model;

/**
 * Stack of Walks.
 */
public interface WalkStack {

	/**
	 * Return this stack to the Factory.
	 * <p><b>Post-condition:</b> this is no more usable.</p>
	 */
	public void returnToFactory();
	
	/**
	 * Return the size of this stack.
	 * @return the size of this stack.
	 */
	public int getSize();

	/**
	 * Returns <tt>true</tt> if this stack is empty.
	 * @return <tt>true</tt> if this stack is empty.
	 */
	public boolean isEmpty();

	/**
	 * Pushes a Walk on this stack.
	 * @param walk a Walk.
	 * @throws IllegalArgumentException if the given Walk is null.
	 */
	public void push(Walk walk);
	
	/**
	 * Pops a Walk from this stack.
	 * @return the poped Walk.
	 * @throws IllegalStateException if the stack is empty.
	 */
	public Walk pop();

	/**
	 * Returns the Walk at the top of this stack or null if it is empty.
	 * @return the Walk at the top of this stack or null if it is empty.
	 */
	public Walk top();

	/**
	 * Return the Walk at a given top relative index in the stack.
	 * <tt>getFromTop(0)</tt> is equivalent to <tt>top()</tt>.
	 * @param index an index in the stack, between 0 and size - 1.
	 * @return the Walk at the index position.
	 * @throws IndexOutOfBoundsException if index < 0 or index >= size.
	 */
	public Walk getFromTop(int index);
	
	/**
	 * Inserts a Walk in this stack at a top relative index.
	 * <tt>insertFromTop(0, w)</tt> is equivalent to <tt>push(w)</tt>.
	 * @param index a top relative index.
	 * @param walk a Walk.
	 * @throws IndexOutOfBoundsException if index < 0 or index >= size.
	 */
	public void insertFromTop(int index, Walk walk);

	/**
	 * Removes and returns the Walk at a given top relative index in the stack.
	 * @param index a top relative index.
	 * @return the removed Walk.
	 * @throws IndexOutOfBoundsException if index < 0 or index >= size.
	 */	
	public Walk removeFromTop(int index);
}
