/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.net.util;

import org.xmloperator.lambda.net.model.Operator;

/**
 * Operator iterator in a closed lambda-net.
 * The iterator modify the marks of all the Operators, to a same value.
 * Iterating must be accomplished entirely.
 * There must be no operation on the net, such as connection/disconnection,
 *    during iterating.
 * <p><b>Pre-condition:</b> all the Operators have the same mark.</p>
 * <p><b>Post-condition:</b> all the Operators have the same mark.</p>
 */
public interface OperatorIterator {

	/**
	 * Returns <tt>true</tt> if there is yet one or more Operator(s) to read.
	 * @return <tt>true</tt> if there is yet one or more Operator(s) to read.
	 */
	public boolean hasNext();

	/**
	 * Returns the next Operator.
	 * @return the next Operator.
	 * @throws IllegalStateException if there is no more Operator.
	 * @throws MarkedDifferentlyException if an Operator has not the same
	 *   mark than the others.
	 * @throws FreePortException if a Port is found free.
	 */
	public Operator next();
}
