/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.shared;

import java.util.Arrays;

/**
 * Simple stack.
 */
public final class Stack {
	static final int INITIAL_LENGTH = 8;
	Object[] objects = new Object[INITIAL_LENGTH];
	int size = 0;

	public Stack() {
	}

	public int getSize() {
		return this.size;
	}

	public boolean isEmpty() {
		return this.size == 0;
	}

	public void clear() {
		Arrays.fill(this.objects, 0, this.size, null);
		this.size = 0;
	}

	public void push(Object object) {
		if (object == null)
			throw new IllegalArgumentException();
		this.ensureOneMore();
		this.objects[this.size] = object;
		this.size++;
	}

	void ensureOneMore() {
		if (this.objects.length == this.size) {
			Object[] newObjects = new Object[2 * objects.length];
			System.arraycopy(this.objects, 0, newObjects, 0, this.size);
			this.objects = newObjects;
		}
	}

	public Object pop() {
		if (this.size == 0)
			return null;
		int index = --this.size;
		Object object = this.objects[index];
		this.objects[index] = null;
		return object;
	}
}
