/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.translate;

import java.io.PrintStream;

import org.xmloperator.lambda.net.model.OperatorFactory;
import org.xmloperator.lambda.net.reduction.NotInNormalFormException;
import org.xmloperator.lambda.net.reduction.ReductionCounter;
import org.xmloperator.lambda.net.reduction.ReductionListener;
import org.xmloperator.lambda.net.util.InteractionStat;
import org.xmloperator.lambda.tree.exception.TermException;
import org.xmloperator.lambda.tree.generate.FullGenerator;
import org.xmloperator.lambda.tree.generate.RandomGenerator;
import org.xmloperator.lambda.tree.generate.TermIterator;
import org.xmloperator.lambda.tree.model.Term;
import org.xmloperator.lambda.tree.model.TermFactory;
import org.xmloperator.lambda.tree.serialize.SerializeUtils;
import org.xmloperator.lambda.tree.serialize.Serializer;
import org.xmloperator.lambda.tree.util.FactoryUtils;

/**
 * Tests using generators.
 */
public abstract class Generate {
	private static final boolean TRACE = false;

	public static void main(String[] args) {
		if (test(System.out, true))
			System.out.println("All is OK");
		System.exit(0);
	}

	private static final TermFactory TERM_FACTORY = Test.TERM_FACTORY;
	private static final OperatorFactory OPERATOR_FACTORY = Test.OPERATOR_FACTORY;
	private static final Serializer SERIALIZER = Test.SERIALIZER;
	private static final InteractionStat INTERACTION_STAT = Test.INTERACTION_STAT;
	private static final boolean IS_VERBOSE = false;

	/**
	 * Executes some Term generations.
	 * @param out the PrintStream to print results. May be null.
	 * @param isVerbose if true then all tests have to be displayed.
	 * @return <tt>true</tt> if all is Ok.
	 */
	public static boolean test(PrintStream out, boolean isVerbose) {
		try {
			isVerbose |= TRACE | IS_VERBOSE;
			boolean isOk = true;
			{
				int maxTreeSize = 5;
				for (int k = 2; k <= maxTreeSize; k++)
					testFullGeneration(out, isVerbose, k);
			}
			{
				long seed = 6821684;
				int treeSize = 16;
				TermIterator iterator =
					new RandomGenerator(TERM_FACTORY, seed, treeSize);
				ReductionCounter reductionCounter = new ReductionCounter(10);
				if (TRACE)
					reductionCounter.nextReductionListener =
						new ReductionTracer(SERIALIZER, true, true, TERM_FACTORY);
				for (int k = 0; k < 40; k++) {
					reductionCounter.clear();
					if (TRACE)
						((ReductionTracer) reductionCounter.nextReductionListener).clear();
					testTerm(iterator.next(), reductionCounter, isVerbose);
				}
			}
			return isOk;
		}
		catch (TermException exception) {
			System.out.println("Generate TermException");
			Term subTerm = exception.getTerm();
			if (SERIALIZER != null && subTerm != null)
				SerializeUtils.printUnderlinedTerm(SERIALIZER, null, subTerm);
			exception.printStackTrace(out);
			return false;
		}
		catch (Exception exception) {
			exception.printStackTrace(out);
			return false;
		}
	}

	private static final void testFullGeneration(
		PrintStream out,
		boolean isVerbose,
		int treeSize) {
		TermIterator iterator = new FullGenerator(TERM_FACTORY, treeSize, true);
		ReductionCounter reductionCounter = new ReductionCounter(10);
		if (TRACE)
			reductionCounter.nextReductionListener =
				new ReductionTracer(SERIALIZER, true, true, TERM_FACTORY);
		int count = 0;
		while (iterator.hasNext()) {
			count++;
			reductionCounter.clear();
			if (TRACE)
				 ((ReductionTracer) reductionCounter.nextReductionListener).clear();
			testTerm(iterator.next(), reductionCounter, isVerbose);
		}
		if (isVerbose)
			System.out.println("Generate(" + treeSize + ") : " + count + " term(s).");
	}

	private static final void testTerm(
		Term term,
		ReductionListener redexListener,
		boolean isVerbose) {
		try {
			Term result =
				Reducer.reduceDeeply(
					term,
					OPERATOR_FACTORY,
					INTERACTION_STAT,
					redexListener);
			if (TRACE)
				SerializeUtils.printTerm(SERIALIZER, result);
			FactoryUtils.returnTerm(result);
		}
		catch (NotInNormalFormException exception) {
		}
		FactoryUtils.returnTerm(term);
	}
}
