/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.translate;

import java.io.PrintWriter;

import org.xmloperator.lambda.net.model.Applicator;
import org.xmloperator.lambda.net.model.Eraser;
import org.xmloperator.lambda.net.model.Port;
import org.xmloperator.lambda.net.reduction.NotInNormalFormException;
import org.xmloperator.lambda.net.reduction.ReductionListener;
import org.xmloperator.lambda.net.util.CloneUtils;
import org.xmloperator.lambda.net.util.ScopeBinderCounter;
import org.xmloperator.lambda.tree.model.Application;
import org.xmloperator.lambda.tree.model.Term;
import org.xmloperator.lambda.tree.model.TermFactory;
import org.xmloperator.lambda.tree.serialize.Serializer;
import org.xmloperator.lambda.tree.util.FactoryUtils;

/**
 * Beta reduction tracer.
 */
public final class ReductionTracer implements ReductionListener {
	private final Serializer serializer;
	private final boolean isDetailled;
	private final boolean isUnderlined;
	private final TermFactory termFactory;
	private final ScopeBinderCounter scopeBinderCounter = new ScopeBinderCounter();
	private int reductionCount = 0;

	public ReductionTracer(
		Serializer serializer,
		boolean isDetailled,
		boolean isUnderlined,
		TermFactory termFactory) {
		this.serializer = serializer;
		this.isDetailled = isDetailled;
		this.isUnderlined = isUnderlined;
		this.termFactory = termFactory;
	}
	
	public void clear() {
		this.reductionCount = 0;
	}

	private void underlineRedexes(
		PrintWriter out,
		Term rootTerm,
		Eraser rootEraser,
		Applicator redexApplicator) {
		WalkingContextTranslater walkingContextTranslater =
			new WalkingContextTranslaterImpl(rootEraser);
		walkingContextTranslater.setRootTerm(rootTerm);
		while (true) {
			walkingContextTranslater.executeStep();
			if (walkingContextTranslater.isReturnedToRoot())
				break;
			Port port = walkingContextTranslater.getCurrentPort();
			if (port.getPortType() == Port.TYPE_APPLICATOR_BODY
				&& port.getOperator() == redexApplicator) {
				Application redex =
					(Application) walkingContextTranslater.getCurrentTerm();
				if (redex == null)
					throw new IllegalStateException();
				this.serializer.underlineRedex(out, rootTerm, redex);
				out.println();
			}
		}
	}

	///////////////////////////////////
	// ReductionListener implementation
	///////////////////////////////////

	public void notifyStartReduction(
		Eraser rootEraser,
		Applicator redexApplicator) throws NotInNormalFormException {
		if (this.serializer == null)
			return;
		PrintWriter out = new PrintWriter(System.out);
		Eraser cloneRootEraser = CloneUtils.cloneNet(rootEraser);
		Term rootTerm = FromNetToTerm.translate(cloneRootEraser, termFactory, null);
		try {
			reductionCount++;
			if (this.isDetailled) {
				this.scopeBinderCounter.count(rootEraser);
				out.print("<");
				out.print(reductionCount);
				out.print(">: ");
				out.println(this.scopeBinderCounter);
				this.serializer.writeTerm(out, rootTerm);
				out.println();
				if (this.isUnderlined)
					underlineRedexes(out, rootTerm, rootEraser, redexApplicator);
			}
			out.flush();
		}
		finally {
			FactoryUtils.returnTerm(rootTerm);
		}
	}
}
