/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.translate;

import org.xmloperator.lambda.net.util.WalkingContext;
import org.xmloperator.lambda.tree.model.Term;

/**
 * Walking context with a translation capability.
 */
public interface WalkingContextTranslater extends WalkingContext {

	/**
	 * Set the root Term.
	 * <p><b>Pre-condition:</b> the walking context must be returned to the root
	 *   <tt>Eraser</tt>.</p>
	 * @param rootTerm a root <tt>Term</tt>. May be null.
	 * @throws IllegalStateException if the walking context is not returned
	 *   to the root <tt>Eraser</tt>.
	 */
	public void setRootTerm(Term rootTerm);
	
	/**
	 * Depending on the currently leaved <tt>Port</tt>, returns the following
	 *   <tt>Term</tt> :
	 * <ul>
	 * <li>Abstractor.Body -> Abstraction</li>
	 * <li>Abstractor.Bind -> EndOfSCope or Leaf</li>
	 * <li>Abstractor.Input -> Abstraction</li>
	 * <li>Applicator.Body -> Application</li>
	 * <li>Applicator.Argument -> Application</li>
	 * <li>Applicator.Input -> Application</li>
	 * <li>Delimitor.Principal -> null</li>
	 * <li>Delimitor.Auxiliary -> null</li>
	 * <li>Duplicator.Output -> null</li>
	 * <li>Duplicator.Input -> null</li>
	 * <li>Eraser.Port -> null</li>
	 * </ul>
	 * @return the <tt>Term</tt> that maps to the walking context or null if the
	 *   root <tt>Term</tt> is null or if the current <tt>Operator</tt> is neither
	 *   an <tt>Abstractor</tt> nor an <tt>Applicator</tt>.
	 */
	public Term getCurrentTerm();
}
