/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.tree.model;

/**
 * Factory of Terms.
 * Terms that are no more used can be returned to it. They can be recycled.
 */
public interface TermFactory {

	/**
	 * Creates a new Leaf, with no parent.
	 * @return the created Leaf.
	 */
	public Leaf createLeaf();

	/**
	 * Creates a new EndOfScope, with no parent and no body.
	 * @return the created EndOfScope.
	 */
	public EndOfScope createEndOfScope();

	/**
	 * Create an Abstraction, with no parent and no body.
	 * @return the created Abstraction.
	 */
	public Abstraction createAbstraction();

	/**
	 * Create an Application, with no parent and no content.
	 * @return the created Application.
	 */
	public Application createApplication();

	/**
	 * Creates and returns a TermStack.
	 * @param acceptNull nulls are accepted within the stack only if this argument
	 *   is <tt>true</tt>.
	 * @return the created TermStack.
	 */
	public TermStack createTermStack(boolean acceptNull);
}
