/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.tree.model;

/**
 * Term stack.
 */
public interface TermStack {

	/**
	 * Returns this TermStack to the Factory.
	 * <p><b>Post-condition:</b> is no more usable.</p>
	 */
	public void returnToFactory();
	
	/**
	 * Returns the size of the stack.
	 * @return the size of the stack.
	 */
	public int getSize();

	/**
	 * Returns <tt>true</tt> if the stack is empty.
	 * @return <tt>true</tt> if the stack is empty.
	 */
	public boolean isEmpty() ;

	/**
	 * Clear the content of this stack.
	 */
	public void clear();
	
	/**
	 * Return <tt>true</tt> if the stack contains the given Term.
	 * @param term a Term.
	 * @return <tt>true</tt> is the stack contains the given Term.
	 */	
	public boolean contains(Term term);
	
	/**
	 * Push a Term at the top of this stack.
	 * @param term a Term to add on top of this stack.
	 * @throws IllegalArgumentException if the given Term is null and this
	 *   stack doesn't accept null Terms.
	 */
	public void push(Term term);

	/**
	 * Returns the Term at the top and pops this stack.
	 * @return the Term at the top and pops this stack.
	 * @throws IllegalStateException if this stack is empty.
	 */
	public Term pop();

	/**
	 * Returns the Term at the top of this stack or null if it is empty.
	 * @return the Term at the top of this stack or null if it is empty.
	 */
	public Term top();
}
