/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.tree.reduction;

import org.xmloperator.lambda.tree.exception.NotInNormalFormException;
import org.xmloperator.lambda.tree.model.Application;
import org.xmloperator.lambda.tree.model.Term;

/**
 * Beta-reduction counter.
 * May be used to stop iterations when reaching a max count.
 */
public class ReductionCounter implements ReductionListener {
	public static final int UNLIMITED = -1;
	
	public ReductionListener nextReductionListener = null;
	private final int maxCount;
	private int count = 0;

	public ReductionCounter(int maxCount) {
		this.maxCount = maxCount;
	}
	
	public int getCount() {
		return this.count;
	}

	public void clear() {
		this.count = 0;
	}

	///////////////////////////////////
	// ReductionListener implementation
	///////////////////////////////////

	public void notifyStartReduction(Application redex)
	  throws NotInNormalFormException {
		if (this.count == this.maxCount)
			throw new NotInNormalFormException();
		ReductionListener reductionListener = this.nextReductionListener;
		if (reductionListener != null)
			reductionListener.notifyStartReduction(redex);
	}
	
	public void notifyEndReduction(Term resultingTerm) {
		this.count++;
		ReductionListener reductionListener = this.nextReductionListener;
		if (reductionListener != null)
			reductionListener.notifyEndReduction(resultingTerm);
	}
}
