/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.tree.reduction;

import java.io.PrintWriter;

import org.xmloperator.lambda.tree.exception.NotInNormalFormException;
import org.xmloperator.lambda.tree.model.Application;
import org.xmloperator.lambda.tree.model.Term;
import org.xmloperator.lambda.tree.serialize.Serializer;
import org.xmloperator.lambda.tree.util.WellFormedness;

/**
 * Beta reduction tracer.
 */
public final class ReductionTracer implements ReductionListener {
	private static final boolean ASSERTION = Test.ASSERTION;

	private final Serializer serializer;
	private final boolean isDetailled;
	private final boolean isUnderlined;
	private Term rootTerm;
	private boolean isRedexRoot = false;
	private int reductionCount = 0;

	public ReductionTracer(
		Serializer serializer,
		boolean isDetailled,
		boolean isUnderlined,
		Term rootTerm) {
		this.serializer = serializer;
		this.isDetailled = isDetailled;
		this.isUnderlined = isUnderlined;
		this.rootTerm = rootTerm;
	}

	///////////////////////////////////
	// ReductionListener implementation
	///////////////////////////////////

	public void notifyStartReduction(Application redex) 
	  throws NotInNormalFormException {
		if (this.serializer == null)
			return;
		PrintWriter out = new PrintWriter(System.out);
		this.isRedexRoot = redex == this.rootTerm;
		reductionCount++;
		String label = "<" + reductionCount + ">";
		if (this.isDetailled) {
			out.println(label);
			this.serializer.writeTerm(out, this.rootTerm);
			out.println();
			if (this.isUnderlined) {
				this.serializer.underlineRedex(out, this.rootTerm, redex);
				out.println();
			}
		}
		out.flush();
		if (ASSERTION)
			WellFormedness.verifyWellFormedness(this.rootTerm);
	}

	public void notifyEndReduction(Term resultingTerm) {
		if (this.serializer == null)
			return;
		PrintWriter out = new PrintWriter(System.out);
		if (this.isRedexRoot)
			this.rootTerm = resultingTerm;
		String label = "<" + reductionCount + ">";
		if (!this.isDetailled)
			out.print(label);
		this.serializer.writeTerm(out, this.rootTerm);
		out.println();
		if (this.isUnderlined) {
			if (!this.isDetailled)
				out.print(label);
			this.serializer.underlineTerm(out, this.rootTerm, resultingTerm);
			out.println();
		}
		out.flush();
		if (ASSERTION)
			WellFormedness.verifyWellFormedness(this.rootTerm);
	}
}
