/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.tree.serialize;

import java.io.IOException;
import java.io.Reader;

import org.xmloperator.lambda.tree.exception.ParsingException;

/**
 * Term parsing at the lexical level.
 */
final class ParsingInput {
	private Reader in;
	private int ch;
	private int location = -1;

	public ParsingInput(Reader in) throws IOException {
		this.in = in;
		this.next();
	}

	public boolean isEmpty() {
		return this.ch < 0;
	}

	public char getChar() {
		return (char) this.ch;
	}

	public int getLocation() {
		return this.location;
	}

	public void next() throws IOException {
		this.ch = this.in.read();
		this.location++;
	}

	public int parseNumber() throws IOException {
		if (this.ch < '0' || this.ch > '9')
			return -1;
		int number = this.ch - '0';
		this.next();
		while (this.ch >= 0) {
			if (this.ch < '0' || this.ch > '9')
				break;
			number = 10 * number + (ch - '0');
			this.next();
		}
		return number;
	}

	public boolean isIdentifier() {
		return this.ch >= 'a' && this.ch <= 'z';
	}
	
	public String parseIdentifier() throws IOException {
		if (this.ch < 'a' || this.ch > 'z')
			throw new ParsingException(this.location);
		StringBuffer sb = new StringBuffer();
		sb.append((char) this.ch);
		this.next();
		while (this.ch >= 0) {
			if (this.ch < '0' || this.ch > '9')
				break;
			sb.append((char) this.ch);
			this.next();
		}
		return sb.toString();
	}
	
	public void parseSeparator(char separator) throws IOException {
		if (this.ch != separator)
			throw new ParsingException(this.location);
		this.next();
	}
}
