/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.tree.serialize;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.Reader;

import org.xmloperator.lambda.tree.model.Application;
import org.xmloperator.lambda.tree.model.Term;
import org.xmloperator.lambda.tree.model.TermFactory;

/**
 * Term serializer.
 */
public interface Serializer {

	/**
	 * Writes a Term.
	 * @param out a PrintWriter.
	 * @param term a Term to write out.
	 * @throws NullPointerException if the Term is broken.
	 */
	public void writeTerm(PrintWriter out, Term term);

	/**
	 * Writes a string value that underline a sub-Term.
	 * @param out a PrintWriter.
	 * @param term a Term.
	 * @param underlinedTerm a sub-Term to underline.
	 * @throws NullPointerException if the Expression is broken or if the
	 *   sub-term is null.
	 * @throws UnsupportedOperationException if this serializer doesn't support
	 *   this operation.
	 */
	public void underlineTerm(PrintWriter out, Term term, Term underlinedTerm)
		throws UnsupportedOperationException;

	/**
	 * Writes a string value that underline a redex.
	 * @param out a PrintWriter.
	 * @param term a Term.
	 * @param underlinedRedex a redex to underline.
	 * @throws NullPointerException if the Expression is broken or if the redex
	 *   is null.
	 * @throws UnsupportedOperationException if this serializer doesn't support
	 *   this operation.
	 */
	public void underlineRedex(
		PrintWriter out,
		Term term,
		Application underlinedRedex)
		throws UnsupportedOperationException;

	/**
	 * Produces a Term from a previous serialization.
	 * @param in a Reader.
	 * @param termFactory the factory for building the Term.
	 * @return the created Term.
	 * @throws IllegalArgumentException if the given TermFactory or the given
	 *   Reader is null.
	 * @throws ParsingException if no Term can be build.
	 */
	public Term readTerm(Reader in, TermFactory termFactory) throws IOException;
}
